##==============================================================================
# CS150: Spring 2010
# MIPS150 Checkpoint 1 Checkoff Test suite
#
# =====================================
# Revision D
# =====================================
#
# =====================================
# Introduction
# =====================================
# The following code will be used to check you off for Checkpoint 1. It is designed 
# to test every instruction in the MIPS150 sub-ISA for correctness. Additionally, it 
# will test basic corner cases such as writing to register '$0,' and has been designed 
# under the assumption that you forward/bypass data on data hazards. The tests do not 
# rigorously test every corner case that can be conceived in the standard MIPS150 
# pipeline. These corner cases will be tested through code that is not released prior 
# to final checkoff. Use this test suite as a way to verify your processor's basic 
# functionality, but continue to write tests of your own to further prove correct operation.
# =====================================
#
# =====================================
# Test Format
# =====================================
# There are a total of 9 tests in the test suite. Each writes a single value (in 
# correct operation) to a different register (registers $16-$25) in the register 
# file.  If a test finishes correctly, it will write '0xDEADBEEF' to the register 
# that it was assigned to.  If it didn't finish correctly, there is no way of knowing 
# what will be written into its register...
#
# The tests (instruction-wise) are designed to serially depend on one another to a 
# certain extent.  In other words, the instructions used in each test will only use the 
# instruction actually being tested plus instructions used in previous tests.  Thus, if 
# your design has a bug, expect the test that tests for that instruction/bug to be broken, 
# and also expect future tests to be broken.  When you find that not everything works 
# correctly, look at the first test that fails, and debug from there.
#
# When all of the tests complete successfully, all registers ($16-$25) will hold the
# value 0xDEADBEEF.  Additionally, register $26 should hold the value 0xC0FFEE0.  If any 
# tests fail, $r26 will some other (undefined) value.
#
# As a general suggestion, use 0xDEADBEEF values to find where in your processor you have 
# bugs.  The tests below are not rigorously commented because they don't do anything 
# sensible or intelligible.  Rather, they have been designed such that if an instruction doesn't 
# execute for some reason in your processor, you will not see 0xDEADBEEF.  For example, 
# the final 0xC0FFEE00 value is derived from the sum of all of the values in registers 
# $16 to $25 + an offset, such that the result is exactly 0xC0FFEE00.  No comments would really
# make that code make any sense at a high level, but it ensures that there is NO WAY 
# (within reason...) you will see 0xC0FFEE00 unless all of the tests pass.
# 
# These tests are provided as a tool to help YOU debug your design and to help US 
# check you off.  These tests might change in the future.  Insofar as checkpoint 1 is 
# concerned, we (the staff) will do our best to ensure that the most up-to-date copy of 
# the tests are on the website.  Bear in mind, however, we will use the most up-to-date 
# copy (that we have made) during the actual checkoff.  As such, if something changes at 
# the last minute (such as the value that is meant to be written to registers $16-$25), 
# we still expect your processor to work correctly.
# =====================================
#
# =====================================
# Test Setup and Checkoff
# =====================================
# For checkoff, you will be required to have completed the following:
#
# 1.) Your MIPS150 processor implementation.
# 2.) A testbench that your processor can run in.  The testbench must be able to:
# 	2a.) Talk to instruction/data memory.
#	2b.) Load this assembly code into instruction memory.
#	2c.) Reset the system and actually run the code.
# 	2d.) Stop when the last instruction has been completely executed.
#	2e.) Dump state to the console for TA inspection.  The staff needs to be able to 
#		visually (at the console) inspect the contents of:
#		2ei.) The register file.
#		2eii.) Data memory.
#		2eiii.) Instruction memory.
# ALL STATE DUMPED TO THE CONSOLE MUST BE SHOWN IN HEX (base 16).
# If you do not remember how to do any of these things in Modelsim, please consult Lab 4.
# Example: If you have a CPU module instance called "cpu" with a RegisterFile module instance 
# called "rfile" with memory implemented with a reg named "M", then you can access the
# value of register i in Verilog using the expression "cpu.rfile.M[i]".
# =====================================
##==============================================================================

##==============================================================================
## Revision History
##==============================================================================
## Revision D - Brandon Myers (2010/3/28)
## Change date to 2010 (note other SMIPS instructions, including or, are in part2,
## another file).
##
## Revision C - Chris Fletcher (2009/3/15) 
##	Removed test for 'or' instruction (as it is not a part of the MIPS150 ISA!)
##
## Revision B - Chris Fletcher (2009/3/15) 
##	Added test 9 (lw/sw).
##
## Revision A - Chris Fletcher (2009/3/15) 
##	Wrote testbench.  8/9 instruction tests ($16-$24) in place.
##==============================================================================

##==============================================================================
# Test setup
##==============================================================================

#
# Initialize some utility registers.
# NOTE: this implicitely tests 'sll,' 'ori,' and data (RAW) hazards.
# If these instructions don't work off the bat, nothing else will work!
#
ori		$13, $0, 0xDEAD
ori		$14, $0, 0xBEEF
ori 	$15, $0, 0xBAD
sll 	$13, $13, 0
sll 	$13, $13, 1
sll 	$13, $13, 2
sll 	$13, $13, 3
sll 	$13, $13, 10
sll 	$14, $14, 0
sll 	$15, $15, 31
sll 	$15, $15, 1

#
# Initialize the result registers 
#
ori		$16, $0, 0x0
ori		$17, $0, 0x0
ori		$18, $0, 0x0
ori		$19, $0, 0x0
ori		$20, $0, 0x0
ori		$21, $0, 0x0
ori		$22, $0, 0x0
ori		$23, $0, 0x0
ori		$24, $0, 0x0
ori		$25, $0, 0x0

##==============================================================================

##==============================================================================
# Test simple (register transfer) instructions
##==============================================================================

#
# Test 16: test 'ori' officially
#
ori 	$16, $13, 0xBEEF		

#
# Test 17: test 'addu' and 'addiu' (without sign extension)
#
addiu 	$17, $14, 0x1FBE	
addiu	$17, $17, 0x0
sll		$17, $17, 16
addu	$17, $17, $14

#
# Test 18 (preliminary): taint register $0
#
# Expect to see errors in $18 and beyond if your processor doesn't handle this 
# line correctly!
#
addu	$0, $13, $14

#
# Test 18: test 'subu', 'addiu' (with sign extension), and writing to register '$0'
#
addiu 	$12, $0, 0xF000		
ori		$18, $0, 0x2152		
sll		$18, $18, 16
ori 	$18, $18, 0x3111
subu 	$18, $12, $18

#
# Test 19: test 'sra' 
#
# Basic shift right arithmetic tests, shifting by different amounts.
#
ori		$12, $0, 0x7FFF
sll		$12, $12, 16
sra		$12, $12, 1
sra		$12, $12, 2
sra		$12, $12, 3
sra		$12, $12, 10
ori		$11, $0, 0xF000
sll		$11, $11, 16
sra		$11, $11, 28
addiu	$11, $11, 0xDEAE
sll		$11, $11, 16
addu	$11, $11, $12
addiu	$19, $11, 0x3EF0

#
# Test 20: test 'sllv' and 'srav'
# 
# These tests will test different shift instructions, shifting back and forth by 
# different amounts.
#
ori		$11, $0, 0xDEAD
ori		$12, $0, 0xBEEF
ori		$10, $0, 0x0
sllv 	$11, $11, $10
ori		$10, $0, 0x1
sllv 	$11, $11, $10	
ori		$10, $0, 0x2
sllv 	$11, $11, $10	
ori		$10, $0, 0x1	
srav 	$11, $11, $10	
ori		$10, $0, 0x3
sllv 	$11, $11, $10	
ori		$10, $0, 0x2
srav 	$11, $11, $10	
ori		$10, $0, 0xa
sllv 	$11, $11, $10	
ori		$10, $0, 0x9
srav 	$11, $11, $10	
ori		$10, $0, 0xc
sllv 	$11, $11, $10	
ori		$10, $0, 0x0
srav 	$11, $11, $10
ori		$20, $11, 0xBEEF
ori		$10, $0, 0x1F
sllv	$12, $12, $10
ori		$10, $0, 0x1
sllv	$12, $12, $10
addu	$20, $20, $12
ori		$11, $0, 0x7EAD
ori		$10, $0, 0x1F
srav	$11, $11, $10
ori		$10, $0, 0x1
srav	$11, $11, $10
addu	$20, $20, $11

#
# Test 21: test 'andi'
#
ori		$11, $0, 0xDEAD 
andi	$11, $11, 0xFFFF	# test masking 'on' bits
sll		$11, $11, 16
ori		$12, $12, 0xBBAD
andi	$12, $12, 0xF000	# test masking 'off' bits
ori		$12, $12, 0x0EEF
addu	$21, $12, $11

#
# Test 22: test 'slt'
#
# Tests what happens when the actual condition is >, <, ==, making sure to 
# disambiguate between the different cases.
#
addiu	$8, $16, -1
slt		$9, $14, $13
sll		$9, $9, 4
slt		$10, $13, $13
sll		$10, $10, 8
slt		$11, $14, $14
sll		$11, $11, 12
slt		$12, $13, $14
addu	$22, $0, $8
addu	$22, $22, $9
addu	$22, $22, $10
addu	$22, $22, $11
addu	$22, $22, $12

##==============================================================================

##==============================================================================
# Test more complicated (control transfer + memory) instructions
##==============================================================================

#
# Test 23: test 'beq' and 'bne' (with delay slots)
#
addiu	$9, $0, 0x0
addiu	$10, $0, 0xa
addiu	$11, $0, 0xf
addiu	$12, $0, 0x0

OuterLoop:	
beq		$9, $10, BranchBreak
addiu	$8, $0, 0x0

InnerLoop:	
addiu	$8, $8, 0x1
bne		$8, $11, InnerLoop
addiu	$12, $12, 0x1
beq		$0, $0, OuterLoop
addiu	$9, $9, 0x1

BranchBreak:
addiu	$23, $16, -150
addu	$23, $23, $12
addu	$23, $23, $8

#
# Test 24: test 'jal' and 'jr' (with delay slots)
#
addu	$2, $0, $0
jal		Foo
addiu	$4, $0, 0xDEAD
slt		$10, $4, $2
beq		$0, $0, JumpBreak
ori		$2, $2, 0x00EF

Foo:
sll		$2, $4, 16
jr		$ra
ori		$2, $2, 0xBE00

JumpBreak: 
addu	$24, $0, $2
addu	$24, $24, $10

#
# Test 25: test 'lw' and 'sw'
#
addiu	$8, $0, 0x1001		# initialize a pointer into the heap
sll		$8, $8, 16
ori		$8, $8, 0x0004
addiu	$9, $0, 0x0			# set temporary variables
addiu	$10, $0, 0x0
addiu	$11, $0, 0x0
sw		$16, 0($8)
lw		$9, 0($8)
sw		$16, -4($8)
bne		$9, $16, LWSWFail
lw		$10, -4($8)
sw		$16, 4($8)
bne		$10, $16, LWSWFail
lw		$11, 4($8)
sll		$0, $0, 0			
bne		$11, $16, LWSWFail
sll		$0, $0, 0	
addu	$25, $0, $16

LWSWFail:

##==============================================================================

##==============================================================================
# Test Teardown: Verify that all of the tests passed
##==============================================================================

addu	$8, $0, $16
addu	$8, $8, $17
addu	$8, $8, $18
addu	$8, $8, $19
addu	$8, $8, $20
addu	$8, $8, $21
addu	$8, $8, $22
addu	$8, $8, $23
addu	$8, $8, $24
addu	$8, $8, $25
ori		$9, $0, 0xE367
sll		$9, $9, 12
ori		$9, $9, 0x8AA
addu	$26, $8, $9			# Write the final result to $26. 
                            # ($r26 == 0xCOFFEE00) -> don't drink anymore coffee 
                            # and go home! You're done :)
##==============================================================================
